/*******************************************************************************
 output_bucket.h - A library for writing memory / file buckets.

 Copyright (C) 2007-2019 CodeShop B.V.
 http://www.code-shop.com

 For licensing see the LICENSE file
******************************************************************************/

/*
 * C-style bucket interface.  C++ clients should should use the C++-style
 * interface in output_bucket.hpp.
 */

#ifndef OUTPUT_BUCKET_H_AKW
#define OUTPUT_BUCKET_H_AKW

#include "mod_streaming_export.h"
#include "buckets_t.h"
#include "mp4_error.h"
#include <inttypes.h>
#include <stdint.h>
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct bucket_t bucket_t;

MP4_DLL_EXPORT uint32_t get_bucket_type(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_heap(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_file(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_http(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_xfrm(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_iobf(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_pipe(bucket_t const* bucket);

MP4_DLL_EXPORT fmp4_result bucket_read(bucket_t* bucket,
                                       uint8_t const** buf, size_t* size);

MP4_DLL_EXPORT fmp4_result bucket_write(bucket_t* bucket,
                                        uint8_t** buf, size_t* size);

MP4_DLL_EXPORT fmp4_result bucket_split(bucket_t* bucket, uint64_t size);

MP4_DLL_EXPORT fmp4_result bucket_copy(bucket_t const* bucket, bucket_t** dst);

MP4_DLL_EXPORT fmp4_result bucket_clone(bucket_t const* bucket, bucket_t** dst);

MP4_DLL_EXPORT bucket_t* bucket_next(bucket_t* bucket);

// Returns UINT64_MAX if unknown
MP4_DLL_EXPORT uint64_t bucket_size(bucket_t const* bucket);

MP4_DLL_EXPORT bucket_t* bucket_heap_create(uint8_t const* buf, uint64_t size);

// utility functions

MP4_DLL_EXPORT extern void bucket_remove(bucket_t* bucket);
MP4_DLL_EXPORT extern void bucket_delete(bucket_t* bucket);

MP4_DLL_EXPORT
fmp4_result bucket_file_read(bucket_t* bucket, char const** filename,
                             uint64_t* offset, uint64_t* size,
			     uint32_t max_size);

#ifdef __cplusplus
} /* extern C definitions */
#endif

#endif // OUTPUT_BUCKET_H_AKW

// End Of File

